# file: sort.py
# author: Bob Muller
#
# CS1101 Computer Science I
#
# Three algorithms for sorting lists.
#
# Insertion Sort --- requires on the order of N^2 units of work.
# Note that this code isn't especially efficient with respect to
# memory usage because it copies the tail of the list to rest each
# time through.
#
def insertionSort(ns):
  def repeat(ms, sorted):
    if ms == []:
      return sorted
    else:
      first = ms[0]
      rest  = ms[1:]
      return repeat(rest, insert(first, sorted))
  return repeat(ns, [])

# Quicksort --- due to C.A.R. Hoare ~1960, requires on the order of
# N log N units of work on average and N^2 units of work in the
# worst case.
#
def partition(pivot, ns):
  smaller = [ i for i in ns if i < pivot]
  larger  = [ i for i in ns if i > pivot]
  return (smaller, larger)

def quicksort(ns):
  if len(ns) < 2:
    return ns
  else:
    pivot = ns[0]
    (smaller, larger) = partition(pivot, ns)
    return quicksort(smaller) + [pivot] + quicksort(larger)

# Mergesort -- due to John von Neumann ~1948, requires on the order
# of N log N units of work on average and in the worst case.
#
def merge(xs, ys):
  if xs == []:
    return ys
  elif ys == []:
    return xs
  else:
    x = xs[0]
    y = ys[0]
    if x < y:
      return [x] + merge(xs[1:], ys)
    else:
      return [y] + merge(xs, ys[1:])

# split : a list -> (a list * a list)
# 
def split(xs):
  n = len(xs)
  left = [ xs[i] for i in range(n / 2)]
  right = [ xs[i] for i in range(n / 2, len(xs))]
  return (left, right)

# mergesort : a list -> a list
#
def mergesort(ns):
  if len(ns) <= 1:
    return ns
  else:
    (left, right) = split(ns)
    return merge(mergesort(left), mergesort(right))



  
